@@ -49,9 +49,10 @@ flag. This is much like the "hg outgoing" command::
 Install
 -------
 
-Just type ``easy_install hgsvn``. If easy_install is not available on your
-computer, download and uncompress the source tarball, then type ``python
-setup.py install``.
+Just type ``easy_install hgsvn``. If easy_install is not available on
+your computer, download and uncompress the source tarball, then type
+``python setup.py install``. Type ``python setup.py install --help``
+for additional options.
 
 *Note:* hgsvn makes use of the ElementTree library. It is bundled by default
 with Python 2.5, and the setup script should install it automatically for you
@@ -150,6 +151,32 @@ repository.
 History
 -------
 
+hgsvn 0.1.9
+-----------
+
+Improvements:
+
+* Improved handling of files containing @ sign in filename (issue52).
+
+* ``hgimportsvn --branch=myname`` imports the SVN repository to a
+  named Mercurial branch other than 'default' (patch by Matthias
+  Benkmann).
+
+* ``hgpullsvn --svn-retry`` retries to retrieve information on flaky
+  connections (patch by Stefanus Du Toit).
+
+* Restrict internal SVN update when running ``hgpushsvn`` to last
+  pushed revision (issue79, by ankon).
+
+* Improved syncing of empty directories (issue77, by ankon, reported
+  by billmichell).
+
+* ``hgpushsvn --keep-author`` keeps author given in hg log when
+  committing to SVN (by wwwjfy).
+
+* Mercurial 1.7 compatibility (issue91, by easye).
+
+
 hgsvn 0.1.8
 -----------
 
@@ -209,6 +236,9 @@ Bug fixes:
 * New sub-directories are now created properly. Issue #46 reported by
   Matt Fowles.
 
+* Several fixes by IanH: issue #64, #94, #95.
+
+
 hgsvn 0.1.7
 -----------
 
@@ -417,7 +447,7 @@ __all__ = []
 
 __author__ = 'Antoine Pitrou, Andi Albrecht'
 __license__ = 'GNU General Public License (version 3 or later)'
-__versioninfo__ = (0, 1, 8)
+__versioninfo__ = (0, 1, 9)
 
 base_version = '.'.join(map(str, __versioninfo__))
 full_version = base_version
