@@ -10,9 +10,10 @@ from hgsvn.common import (
     hg_commit_from_svn_log_entry, hg_exclude_options,
     get_svn_rev_from_hg, hg_switch_branch, hg_is_clean,
     check_for_applied_patches,
+    once_or_more,
 )
 from hgsvn.svnclient import (
-    get_svn_info, get_svn_versioned_files, iter_svn_log_entries, run_svn_log,
+    get_svn_info, get_svn_versioned_files, iter_svn_log_entries,
     get_svn_client_version, get_svn_status
 )
 from hgsvn.errors import (
@@ -53,7 +54,7 @@ def detect_overwritten_svn_branch(wc_url, svn_rev):
         raise OverwrittenSVNBranch(msg)
 
 
-def pull_svn_rev(log_entry, current_rev, svn_wc, wc_url, wc_base):
+def pull_svn_rev(log_entry, current_rev, svn_wc, wc_url, wc_base, retry):
     """
     Pull SVN changes from the given log entry.
     Returns the new SVN revision. If an exception occurs, it will rollback
@@ -67,6 +68,9 @@ def pull_svn_rev(log_entry, current_rev, svn_wc, wc_url, wc_base):
     changed_paths = []
     unrelated_paths = []
     replaced_paths = {}
+
+    # 1. Prepare for the `svn up` changes that are pulled in the second step
+    #    by analyzing log_entry for the changeset
     for d in log_entry['changed_paths']:
         # e.g. u'/branches/xmpp-subprotocols-2178-2/twisted/words/test/test_jabberxmlstream.py'
         p = d['path']
@@ -108,7 +112,13 @@ def pull_svn_rev(log_entry, current_rev, svn_wc, wc_url, wc_base):
         if d['action'] == 'A':
             added_paths.append(p)
         elif d['action'] == 'D':
-            removed_paths.append(p)
+            # Same as above: unfold directories into their affected files
+            if not os.path.isdir(p):
+                removed_paths.append(p)
+            else:
+                r = run_hg(["st", "-nc"], [p], output_is_locale_encoding=True)
+                for f in r.splitlines():
+                    removed_paths.append(f)
         elif d['action'] == 'R':
             # (R)eplaced directories can have added and removed files without
             # them being mentioned in the SVN log => we must detect those files
@@ -122,13 +132,14 @@ def pull_svn_rev(log_entry, current_rev, svn_wc, wc_url, wc_base):
                 # is safest.
                 added_paths.append(p)
 
-    # Update SVN + add/remove/commit hg
+    # 2. Update SVN + add/remove/commit hg
     try:
         if changed_paths:
             args = ["up", "--ignore-externals"]
             if get_svn_client_version() >= (1, 5):
                 args.extend(['--accept', 'postpone'])
-            run_svn(args + ["-r", svn_rev, svn_wc])
+            ui.status('Attempting to update to revision %s...', str(svn_rev))
+            once_or_more("SVN update", retry, run_svn, args + ["-r", svn_rev, svn_wc])
             conflicts = []
             for status_entry in get_svn_status('.'):
                 if status_entry['status'] == 'conflicted':
@@ -161,7 +172,15 @@ def pull_svn_rev(log_entry, current_rev, svn_wc, wc_url, wc_base):
                         raise
                     run_hg(["add"], [new])
             if removed_paths:
-                run_hg(["remove", "-A"], removed_paths)
+                try: 
+                    for file_path in removed_paths:
+                       if os.path.exists(file_path):
+                           run_hg(["remove", "-A"], file_path)
+                except (ExternalCommandFailed), e:
+                    if str(e).find("file is untracked") > 0:
+                        ui.status("Ignoring warnings about untracked files: '%s'" % str(e), level=ui.VERBOSE)
+                    else:
+                        raise
             hg_commit_from_svn_log_entry(log_entry)
         elif unrelated_paths:
             detect_overwritten_svn_branch(wc_url, svn_rev)
@@ -185,12 +204,9 @@ def pull_svn_rev(log_entry, current_rev, svn_wc, wc_url, wc_base):
 def real_main(options, args):
     if check_for_applied_patches():
         print ("There are applied mq patches. Put them aside before "
-               "running hgpsuhsvn.")
+               "running hgpullsvn.")
         return 1
     svn_wc = "."
-    svn_peg_wc = svn_wc
-    if options.svn_peg:
-       svn_peg_wc += "@" + str(options.svn_peg)
 
     # Get SVN info
     svn_info = get_svn_info('.')
@@ -200,13 +216,16 @@ def real_main(options, args):
     repos_url = svn_info['repos_url']
     # e.g. u'svn://svn.twistedmatrix.com/svn/Twisted/branches/xmpp-subprotocols-2178-2'
     wc_url = svn_info['url']
-    if options.svn_peg:
-        svn_peg_wc = wc_url + "@" + str(options.svn_peg)
     assert wc_url.startswith(repos_url)
     # e.g. u'/branches/xmpp-subprotocols-2178-2'
     wc_base = wc_url[len(repos_url):]
     # e.g. 'xmpp-subprotocols-2178-2'
     svn_branch = wc_url.split("/")[-1]
+
+    # if --branch was passed, override the branch name derived above
+    if options.svn_branch:
+        svn_branch = options.svn_branch
+
     if options.svn_peg:
        wc_url += "@" + str(options.svn_peg)
 
@@ -217,20 +236,24 @@ def real_main(options, args):
         ui.status("No revisions after %s in SVN repo, nothing to do",
                   svn_greatest_rev)
         return
+    elif options.svn_rev != None:
+        if options.svn_rev < next_rev:
+            ui.status("All revisions up to %s are already pulled in, "
+                      "nothing to do",
+                      options.svn_rev)
+            return
+        svn_greatest_rev = options.svn_rev
 
     # Show incoming changes if in dry-run mode.
     if options.dryrun:
         ui.status("Incoming SVN revisions:")
-        for entry in run_svn_log(repos_url, next_rev, svn_greatest_rev,
-                                 100):
+        for entry in iter_svn_log_entries(wc_url, next_rev, svn_greatest_rev,
+                                          options.svnretry):
             if entry["message"]:
                 msg = entry["message"].splitlines()[0].strip()
             else:
                 msg = ""
             line = "[%d] %s (%s)" % (entry["revision"], msg, entry["author"])
-            # TODO: Line stripping should be done by some simple UI interface.
-            if len(line) > 79:
-                line = line[:77]+"..."
             ui.status(line)
         return
 
@@ -258,7 +281,7 @@ def real_main(options, args):
         next_rev = hgsvn_rev + 1
 
     # Reset working branch to last svn head to have a clean and linear SVN
-    # histoy.
+    # history.
     heads_before = None
     if hgsvn_rev is None:
         heads_before = run_hg(["heads", "--template",
@@ -266,13 +289,13 @@ def real_main(options, args):
         run_hg(["update", "-C", "svn.%d" % current_rev])
 
     # Load SVN log starting from current rev
-    it_log_entries = iter_svn_log_entries(svn_peg_wc, next_rev, svn_greatest_rev)
+    it_log_entries = iter_svn_log_entries(wc_url, next_rev, svn_greatest_rev, options.svnretry)
 
     try:
         try:
             for log_entry in it_log_entries:
                 current_rev = pull_svn_rev(log_entry, current_rev,
-                                           svn_wc, wc_url, wc_base)
+                                           svn_wc, wc_url, wc_base, options.svnretry)
                 if current_rev is None:
                     return 1
                 ui.status("Pulled r%d %s (%s)", log_entry["revision"],
@@ -302,13 +325,20 @@ def real_main(options, args):
 
 def main():
     # Defined as entry point. Must be callable without arguments.
-    usage = "usage: %prog [-p SVN peg rev] [--help]"
+    usage = "usage: %prog [-p SVN_PEG] [--help]"
     parser = OptionParser(usage)
+    parser.add_option("-r", type="int", dest="svn_rev",
+       help="limit pull up to specified revision")
     parser.add_option("-p", "--svn-peg", type="int", dest="svn_peg",
        help="SVN peg revision to locate checkout URL")
     parser.add_option("-n", "--dry-run", dest="dryrun", default=False,
                       action="store_true",
                       help="show incoming changesets without pulling them")
+    parser.add_option("--branch", type="string", dest="svn_branch",
+        help="override branch name (defaults to last path component of <SVN URL>)")
+    parser.add_option("--svn-retry", dest="svnretry", default=False,
+                      action="store_true",
+                      help="retry SVN update command on failure")
     (options, args) = run_parser(parser, __doc__)
     if args:
         display_parser_error(parser, "incorrect number of arguments")
