@@ -14,7 +14,7 @@ from optparse import OptionParser
 from hgsvn import ui
 from hgsvn.common import (
     run_hg, run_svn, hg_switch_branch, hgsvn_private_dir,
-    check_for_applied_patches,
+    check_for_applied_patches, get_encoding
     )
 from hgsvn.errors import ExternalCommandFailed, EmptySVNLog
 from hgsvn.run.common import run_parser, display_parser_error
@@ -45,7 +45,7 @@ def get_hg_cset(rev_string):
     """
     Given a string identifying an hg revision, get the canonical changeset ID.
     """
-    args = ["log", "--template", "{rev}:{node|short}\n", "-r", rev_string]
+    args = ["log", "--template", r"{rev}:{node|short}\n", "-r", rev_string]
     return run_hg(args).strip()
 
 def get_hg_revs(first_rev, svn_branch, last_rev="tip"):
@@ -53,8 +53,8 @@ def get_hg_revs(first_rev, svn_branch, last_rev="tip"):
     Get a chronological list of revisions (changeset IDs) between the two
     revisions (included).
     """
-    args = ["log", "--template", '{rev}:{node|short}\n', "-b", svn_branch,
-            '--limit', '1000',
+    args = ["log", "--template", r'{rev}:{node|short}\n', "-b", svn_branch,
+            '--limit', '1000', '--follow',
             "-r", "%s:%s" % (strip_hg_rev(first_rev),
                              strip_hg_rev(last_rev))]
     out = run_hg(args)
@@ -113,7 +113,7 @@ def get_ordered_dirs(l):
 
 def get_hg_csets_description(start_rev, end_rev):
     """Get description of a given changeset range."""
-    return run_hg(["log", "--template", "{desc|strip}\n", "--rev",
+    return run_hg(["log", "--template", r"{desc|strip}\n", "--follow", "--rev",
                    start_rev+":"+end_rev, "--prune", start_rev])
 
 def get_hg_cset_description(rev_string):
@@ -123,7 +123,7 @@ def get_hg_cset_description(rev_string):
 def get_hg_log(start_rev, end_rev):
     """Get log messages for given changeset range."""
 
-    log_args=["log", "--verbose", "--rev", start_rev+":"+end_rev, "--prune", start_rev]
+    log_args=["log", "--verbose", "--follow", "--rev", start_rev+":"+end_rev, "--prune", start_rev]
     return run_hg(log_args)
 
 def get_svn_subdirs(top_dir):
@@ -150,7 +150,7 @@ def strip_nested_removes(targets):
     clean_targets = []
     for target in targets:
         try:
-            run_svn(['info', '--xml', target])
+            run_svn(['info', '--xml', target], mask_atsign=True)
         except ExternalCommandFailed, err:
             ui.status(str(err), level=ui.DEBUG)
             continue
@@ -168,7 +168,8 @@ def adjust_executable_property(files):
     execflags = stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH
     svn_map = {}
     for fname in files:
-        if run_svn(['propget', 'svn:executable', fname]).strip():
+        if run_svn(['propget', 'svn:executable', fname],
+                   mask_atsign=True).strip():
             svn_map[fname] = True
         else:
             svn_map[fname] = False
@@ -176,9 +177,10 @@ def adjust_executable_property(files):
         m = os.stat(fname).st_mode & 0777
         is_exec = bool(m & execflags)
         if is_exec and not svn_map[fname]:
-            run_svn(['propset', 'svn:executable', 'ON', fname])
+            run_svn(['propset', 'svn:executable', 'ON', fname],
+                    mask_atsign=True)
         elif not is_exec and svn_map[fname]:
-            run_svn(['propdel', 'svn:executable', fname])
+            run_svn(['propdel', 'svn:executable', fname], mask_atsign=True)
 
 def do_svn_copy(src, dest):
     """
@@ -199,16 +201,20 @@ def do_svn_copy(src, dest):
             # Create svn subdirectories if needed
             # We know that subdirectories themselves are present
             # as dest is present
+            pattern = re.compile(r"[/\\]")
             pos = 0
             while(True):
-                pos = dest.find('/', pos + 1)
-                if pos == -1:
+                match = pattern.search(dest, pos + 1)
+                if match == None:
                     break
 
-                run_svn(['add'], [dest[:pos]])
+                pos = match.start()
+                run_svn(['add', '--depth=empty'], [dest[:pos]],
+                        mask_atsign=True)
+                pos = match.end() - 1
 
             # Do the copy itself
-            run_svn(['copy', src, dest])
+            run_svn(['copy', src, dest], mask_atsign=True)
         except ExternalCommandFailed:
             # Revert rename
             os.rename(src, dest)
@@ -219,7 +225,7 @@ def do_svn_copy(src, dest):
         if(backup_src):
             os.rename(backup_src, src)
 
-def hg_push_svn(start_rev, end_rev, edit):
+def hg_push_svn(start_rev, end_rev, edit, username, password, cache):
     """
     Commit the changes between two hg revisions into the SVN repository.
     Returns the SVN revision object, or None if nothing needed checking in.
@@ -227,22 +233,24 @@ def hg_push_svn(start_rev, end_rev, edit):
     added, removed, modified, copied = get_hg_changes(start_rev+':'+end_rev)
 
     # Before replicating changes revert directory to previous state...
-    run_hg(['revert', '--all', '--no-backup'])
+    run_hg(['revert', '--all', '--no-backup', '-r', end_rev])
 
     # ... and restore .svn directories, if we lost some of them due to removes
     run_svn(['revert', '--recursive', '.'])
 
+    # Do the rest over up-to-date working copy
+    # Issue 94 - moved this line to prevent do_svn_copy crashing when its not the first changeset
+    run_hg(["up", "-C", end_rev])
+
     # Record copyies into svn
     for dest, src in copied.iteritems():
         do_svn_copy(src,dest)
 
-    # As copies are recorded we can do the rest over up-to-date working copy
-    run_hg(["up", "-C", end_rev])
-
     # Add new files and dirs
     if added:
         bulk_args = get_ordered_dirs(added) + added
-        run_svn(["add"], bulk_args)
+        run_svn(["add", "--depth=empty"], bulk_args,
+                mask_atsign=True)
     removed = cleanup_svn_unversioned(removed)
     modified = cleanup_svn_unversioned(modified)
     # Remove old files and empty dirs
@@ -254,7 +262,7 @@ def hg_push_svn(start_rev, end_rev, edit):
         # instead of "svn rm foo/bar foo" it should be "svn rm foo".
         # See issue15.
         svn_removed = strip_nested_removes(removed + empty_dirs)
-        run_svn(["rm"], svn_removed)
+        run_svn(["rm"], svn_removed, mask_atsign=True)
     if added or removed or modified:
         svn_sep_line = "--This line, and those below, will be ignored--\n"
         adjust_executable_property(added+modified)  # issue24
@@ -265,7 +273,8 @@ def hg_push_svn(start_rev, end_rev, edit):
         lines.append("To cancel commit just delete text in top message part")
         lines.append("")
         lines.append("Changes to be committed into svn:")
-        lines.extend(run_svn(["st", "-q"]).splitlines())
+        lines.extend([item.decode("utf-8")
+                      for item in run_svn(["st", "-q"]).splitlines()])
         lines.append("")
         lines.append(("These changes are produced from the following "
                       "Hg changesets:"))
@@ -301,17 +310,25 @@ def hg_push_svn(start_rev, end_rev, edit):
                     raise EmptySVNLog("Commit cancelled by user\n")
 
             svn_rev = None
-            old_lc_all = os.getenv('LC_ALL')
-            os.environ['LC_ALL'] = 'C'
-            try:
-                out = run_svn(["commit", "-F", fname])
-            finally:
-                if old_lc_all:
-                    os.environ['LC_ALL'] = old_lc_all
-                else:
-                    del os.environ['LC_ALL']
-            for line in out.splitlines():
-                match = re.search("^Committed revision (\d*)", line)
+            svn_commands = ["commit", "-F", fname, "--encoding", get_encoding()]
+            if username is not None:
+                svn_commands += ["--username", username]
+            if password is not None:
+                svn_commands += ["--password", password]
+            if cache:
+                svn_commands.append("--no-auth-cache")
+            out = run_svn(svn_commands)
+
+            outlines = out.splitlines()
+            outlines.reverse()
+            for line in outlines:
+                # one of the last lines holds the revision.
+                # The previous approach to set LC_ALL to C and search
+                # for "Committed revision XXX" doesn't work since
+                # svn is unable to convert filenames containing special
+                # chars:
+                # http://svnbook.red-bean.com/en/1.2/svn.advanced.l10n.html
+                match = re.search("(\d+)", line)
                 if match:
                     svn_rev = int(match.group(1))
                     break
@@ -332,7 +349,7 @@ def real_main(options, args):
         return 1
     if check_for_applied_patches():
         print ("There are applied mq patches. Put them aside before "
-               "running hgpsuhsvn.")
+               "running hgpushsvn.")
         return 1
     svn_info = get_svn_info(".")
     svn_current_rev = svn_info["last_changed_rev"]
@@ -390,9 +407,20 @@ def real_main(options, args):
             for prev_rev, next_rev in get_pairs(hg_revs):
                 if not options.dryrun:
                     if not options.edit:
-                        ui.status("Committing changes up to revision %s", get_hg_cset(next_rev))
-                    svn_rev = hg_push_svn(prev_rev, next_rev, edit=options.edit)
+                        ui.status("Committing changes up to revision %s",
+                                    get_hg_cset(next_rev))
+                    username = options.username
+                    if options.keep_author:
+                        username = run_hg(["log", "-r", next_rev,
+                                            "--template", "{author}"])
+                    svn_rev = hg_push_svn(prev_rev, next_rev,
+                                            edit=options.edit,
+                                            username=username,
+                                            password=options.password,
+                                            cache=options.cache)
                     if svn_rev:
+                        # Issue 95 - added update to prevent add/modify/delete crash
+                        run_svn(["up"])
                         map_svn_rev_to_hg(svn_rev, next_rev, local=True)
                         pushed_svn_revs.append(svn_rev)
                 else:
@@ -415,7 +443,7 @@ def real_main(options, args):
             msg = "Pushed one revision to SVN: "
         else:
             msg = "Pushed %d revisions to SVN: " % len(pushed_svn_revs)
-        run_svn(["up"])
+        run_svn(["up", "-r", pushed_svn_revs[-1]])
         ui.status("%s %s", msg, ", ".join(str(x) for x in pushed_svn_revs))
         for line in run_hg(["st"]).splitlines():
             if line.startswith("M"):
@@ -442,6 +470,18 @@ def main():
     parser.add_option("-e", "--edit", default=False, action="store_true",
                       dest="edit",
                       help="edit commit message using external editor")
+    parser.add_option("-u", "--username", default=None, action="store", type="string",
+                      dest="username",
+                      help="specify a username ARG as same as svn --username")
+    parser.add_option("-p", "--password", default=None, action="store", type="string",
+                      dest="password",
+                      help="specify a password ARG as same as svn --password")
+    parser.add_option("--no-auth-cache", default=False, action="store_true",
+                      dest="cache",
+                      help="Prevents caching of authentication information")
+    parser.add_option("--keep-author", default=False, action="store_true",
+                      dest="keep_author",
+                      help="keep the author when committing to SVN")
     (options, args) = run_parser(parser, __doc__)
     if args:
         display_parser_error(parser, "incorrect number of arguments")
