@@ -1,6 +1,6 @@
 
 from hgsvn import ui
-from hgsvn.common import run_svn
+from hgsvn.common import (run_svn, once_or_more)
 from hgsvn.errors import EmptySVNLog
 
 import os
@@ -90,7 +90,8 @@ def parse_svn_log_xml(xml_string):
         author = entry.find('author')
         date = entry.find('date')
         msg = entry.find('msg')
-        d['author'] = author is not None and author.text or None
+        # Issue 64 - modified to prevent crashes on svn log entries with "No author"
+        d['author'] = author is not None and author.text or "No author"
         if date is not None:
             d['date'] = svn_date_to_timestamp(date.text)
         else:
@@ -110,7 +111,7 @@ def parse_svn_log_xml(xml_string):
         l.append(d)
     return l
 
-def parse_svn_status_xml(xml_string, base_dir=None):
+def parse_svn_status_xml(xml_string, base_dir=None, ignore_externals=False):
     """
     Parse the XML output from an "svn status" command and extract useful info
     as a list of dicts (one per status entry).
@@ -129,6 +130,8 @@ def parse_svn_status_xml(xml_string, base_dir=None):
         d['path'] = path
         wc_status = entry.find('wc-status')
         if wc_status.get('item') == 'external':
+            if ignore_externals:
+                continue
             d['type'] = 'external'
         elif wc_status.get('revision') is not None:
             d['type'] = 'normal'
@@ -162,7 +165,7 @@ def svn_checkout(svn_url, checkout_dir, rev_number=None):
     args += [svn_url, checkout_dir]
     return run_svn(svn_checkout_args + args)
 
-def run_svn_log(svn_url_or_wc, rev_start, rev_end, limit, stop_on_copy=False):
+def run_svn_log(svn_url, rev_start, rev_end, limit, stop_on_copy=False):
     """
     Fetch up to 'limit' SVN log entries between the given revisions.
     """
@@ -170,7 +173,7 @@ def run_svn_log(svn_url_or_wc, rev_start, rev_end, limit, stop_on_copy=False):
         args = ['--stop-on-copy']
     else:
         args = []
-    args += ['-r', '%s:%s' % (rev_start, rev_end), '--limit', limit, svn_url_or_wc]
+    args += ['-r', '%s:%s' % (rev_start, rev_end), '--limit', limit, svn_url]
     xml_string = run_svn(svn_log_args + args)
     return parse_svn_log_xml(xml_string)
 
@@ -186,7 +189,7 @@ def get_svn_status(svn_wc, quiet=False):
     else:
         args += ['-v']
     xml_string = run_svn(svn_status_args + args)
-    return parse_svn_status_xml(xml_string, svn_wc)
+    return parse_svn_status_xml(xml_string, svn_wc, ignore_externals=True)
 
 def get_svn_versioned_files(svn_wc):
     """
@@ -234,7 +237,7 @@ def get_last_svn_log_entry(svn_url, rev_start, rev_end):
 log_duration_threshold = 10.0
 log_min_chunk_length = 10
 
-def iter_svn_log_entries(svn_url, first_rev, last_rev):
+def iter_svn_log_entries(svn_url, first_rev, last_rev, retry):
     """
     Iterate over SVN log entries between first_rev and last_rev.
 
@@ -243,23 +246,28 @@ def iter_svn_log_entries(svn_url, first_rev, last_rev):
     """
     cur_rev = first_rev
     chunk_length = log_min_chunk_length
-    chunk_interval_factor = 1.0
+    first_run = True
     while last_rev == "HEAD" or cur_rev <= last_rev:
         start_t = time.time()
-        stop_rev = min(last_rev, cur_rev + int(chunk_length * chunk_interval_factor))
-        ui.status("Fetching SVN log for revisions %d-%d...", cur_rev, stop_rev,
-                  level=ui.VERBOSE)
-        entries = run_svn_log(svn_url, cur_rev, stop_rev, chunk_length)
+        stop_rev = min(last_rev, cur_rev + chunk_length)
+        ui.status("Fetching %s SVN log entries starting from revision %d...",
+                  chunk_length, cur_rev, level=ui.VERBOSE)
+        entries = once_or_more("Fetching SVN log", retry, run_svn_log, svn_url,
+                               cur_rev, "HEAD", chunk_length)
         duration = time.time() - start_t
+        if not first_run:
+            # skip first revision on subsequent runs, as it is overlapped
+            entries.pop(0)
+        first_run = False
         if not entries:
-            if stop_rev == last_rev:
-                break
-            cur_rev = stop_rev + 1
-            chunk_interval_factor *= 2.0
-            continue
+            break
         for e in entries:
+            if e['revision'] > last_rev:
+                break
             yield e
-        cur_rev = e['revision'] + 1
+        if e['revision'] >= last_rev:
+            break
+        cur_rev = e['revision']
         # Adapt chunk length based on measured request duration
         if duration < log_duration_threshold:
             chunk_length = int(chunk_length * 2.0)
